import os

import numpy as np
import tensorflow as tf
import logging



def load_array(path):
    """
    reads the first array saved in the npz file
    :param path: path to npz file
    :return: first numpy array in the file
    """
    a = np.load(path)
    b = a[a.files[0]]
    return b

def read_file(filepath):
    """
    reads the image saved in filepath and converts to tensor. Supports both .npz and .png files.
    :param filepath: path to image
    :return: float 32 tensor containing the image with shape(width, height, channel)
    """
    image = None
    type = filepath.split('/')[-1].split('.')[-1]
    if type == "png":
        file = tf.io.read_file(filepath)
        image = tf.io.decode_png(file)
    elif type == "npz":
        image = load_array(filepath)
        image = np.reshape(image, (image.shape[0], image.shape[1],1))
        image = tf.convert_to_tensor(image)

    else:
        raise Exception("{} is Unsupported File Type".format(type))
    image = tf.image.convert_image_dtype(image, tf.float32)
    return image
def parse_input(input_path, gt_path, image_name):
    """
    reads the image with the name image_name from input_path and the corresponding ground truth from gt_path
    :param input_path: path to the folder with the blured images
    :param gt_path: path to the folder with ground truth images
    :param image_name: name of the image
    :return: (input, gt): two float32 tensors with shape (width, height, channel)
    """
    image = read_file(input_path + image_name)
    gt = read_file(gt_path + image_name)
    return image, gt
def move_dataset_to_mem(input_path,gt_path,max =-1,type="npz"):
    """
    loads images from storage to ram one by one using the given folders
    :param input_path: path to the folder with the blured images
    :param gt_path: path to the folder with ground truth images
    :param max: number of images to load set to -1 to load all the images in input_path
    :param type: image type (npz or png)
    :return: tf dataset with objects (input, gt). Both are tensors with shape (width, height, channel)
    """
    images = []
    gts = []
    paths = [i for i in os.listdir(input_path) if i.split(".")[-1] == type][:max]
    j = 1
    for i in paths:
        logging.info("Reading {} of {} in {}".format(j,len(paths),input_path))
        images.append(read_file(input_path+i))
        gts.append(read_file(gt_path+i))
        j +=1

    mean = tf.reduce_mean(images)
    var = tf.math.reduce_variance(images)
    return tf.data.Dataset.from_tensor_slices((images,gts)), (mean,var)
@tf.function
def randomize(shape, image, gt):
    """
    Randomize the dataset. Randomization is reached by cropping a random part of the image and randomly flipping it.
    This is applied both to image and gt simultaneously. Can be used with tf.data.Dataset.map
    :param shape: sets the output shape of the random crop. Should be either (height, width, 1) or None to disable cropping.
    :param image: tensor of the input
    :param gt: tensor of the ground truth
    :return: randomized (input, gt) tensors with shape (width, height, channel)
    """
    i,g = image,gt
    if shape is not None:
        seed = tf.random.uniform([2], maxval=13467796,dtype=tf.int32)
        i = tf.image.stateless_random_crop(i, shape, seed)
        g = tf.image.stateless_random_crop(g, shape, seed)

    seed = tf.random.uniform([2], maxval=13467796,dtype=tf.int32)
    i = tf.image.stateless_random_flip_left_right(i, seed)
    g = tf.image.stateless_random_flip_left_right(g, seed)

    seed = tf.random.uniform([2], maxval=13467796,dtype=tf.int32)
    i = tf.image.stateless_random_flip_up_down(i, seed)
    g = tf.image.stateless_random_flip_up_down(g, seed)

    return i, g

